import 'package:flutter/material.dart';
import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

import 'package:snake/config/default_activity_settings.dart';
import 'package:snake/models/settings/settings_activity.dart';

part 'settings_activity_state.dart';

class ActivitySettingsCubit extends HydratedCubit<ActivitySettingsState> {
  ActivitySettingsCubit()
      : super(ActivitySettingsState(settings: ActivitySettings.createDefault()));

  void setValues({
    String? difficultyLevel,
    String? boardSize,
  }) {
    emit(
      ActivitySettingsState(
        settings: ActivitySettings(
          difficultyLevel: difficultyLevel ?? state.settings.difficultyLevel,
          parameterSize: boardSize ?? state.settings.parameterSize,
        ),
      ),
    );
  }

  String getParameterValue(String code) {
    switch (code) {
      case DefaultActivitySettings.parameterCodeDifficultyLevel:
        return ActivitySettings.getLevelValueFromUnsafe(state.settings.difficultyLevel);
      case DefaultActivitySettings.parameterCodeBoardSize:
        return ActivitySettings.getSizeValueFromUnsafe(state.settings.parameterSize);
    }

    return '';
  }

  void setParameterValue(String code, String value) {
    final String difficultyLevel =
        (code == DefaultActivitySettings.parameterCodeDifficultyLevel)
            ? value
            : getParameterValue(DefaultActivitySettings.parameterCodeDifficultyLevel);
    final String boardSize = (code == DefaultActivitySettings.parameterCodeBoardSize)
        ? value
        : getParameterValue(DefaultActivitySettings.parameterCodeBoardSize);

    setValues(
      difficultyLevel: difficultyLevel,
      boardSize: boardSize,
    );
  }

  @override
  ActivitySettingsState? fromJson(Map<String, dynamic> json) {
    final String difficultyLevel =
        json[DefaultActivitySettings.parameterCodeDifficultyLevel] as String;
    final String boardSize = json[DefaultActivitySettings.parameterCodeBoardSize] as String;

    return ActivitySettingsState(
      settings: ActivitySettings(
        difficultyLevel: difficultyLevel,
        parameterSize: boardSize,
      ),
    );
  }

  @override
  Map<String, dynamic>? toJson(ActivitySettingsState state) {
    return <String, dynamic>{
      DefaultActivitySettings.parameterCodeDifficultyLevel: state.settings.difficultyLevel,
      DefaultActivitySettings.parameterCodeBoardSize: state.settings.parameterSize,
    };
  }
}
